/**
 * DBus bridge to the plasmoid.
 *
 * @license GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author Nikita Melnichenko [http://nikita.melnichenko.name]
 * @copyright Copyright 2010, Nikita Melnichenko
 */

/**
 * Constructor.
 *
 * DBus handler that talks with Make progress! engine.
 * Should be a singleton.
 */
function JDBus()
{
	$.debug_print("@@@ JDBus: contructor");

	// all sources of the engine will use the following prefix
	this.engine_tag = "[jdbus] ";
	// init engine
	this.engine = dataEngine("name-melnichenko-nikita-make-progress-engine");
	if (this.engine && this.engine.valid)
	{
		// connect to source-related signals
		var $this = this;
		this.engine.sourceAdded.connect(function(name) { $this.sourceAdded(name); });
		this.engine.sourceRemoved.connect(function(name) { $this.sourceRemoved(name); });

		$.debug_print("@@@ JDBus: initial sources are: " + this.engine.sources);
	}
	else
	{
		$.message("DBus support is disabled because the engine is not valid!");
		$.message("For more information about DBus subsystem visit the homepage.");
	}

	// flags
	this.is_disabled = false; // is disabled from ui?
	this.is_registered = false; // is registered in the engine?
	this.accepting_items = false; // is the engine accepting new items?

	// define signals
	this.activated = new JSignal(this); // instance has been registered
}

/// supported progress properties that could be exported and imported
JDBus.progress_props = [ "minimum", "maximum", "value", "active", "speed" ];

/**
 * Is handler valid?
 *
 * @return bool true, if handler is valid
 */
JDBus.prototype.isValid = function()
{
	return this.engine && this.engine.valid && !this.is_disabled;
};

/**
 * Is handler active, i.e. valid and registered?
 *
 * @return bool true, if handler is active
 */
JDBus.prototype.isActive = function()
{
	return this.isValid() && this.is_registered;
};

/**
 * Slot that handles new sources.
 *
 * @param string name name of the source
 */
JDBus.prototype.sourceAdded = function(name)
{
	$.debug_print("@@@ JDBus: sourceAdded(" + name + ")");

	// connect to progress factory once it appeares
	if (name == this.engine_tag + "progress factory" && !this.accepting_items)
	{
		this.accepting_items = true;
		this.engine.connectSource(this.engine_tag + "progress factory", plasmoid);
	}
};

/**
 * Slot that handles removed sources.
 *
 * @param string name name of the source
 */
JDBus.prototype.sourceRemoved = function(name)
{
	$.debug_print("@@@ JDBus: sourceRemoved(" + name + ")");
};

/**
 * dataUpdated slot for events from the dbus engine.
 *
 * JS API v1 allows to connect source to plasmoid.dataUpdated only.
 * To separate engine replies from other replies we check prefix that used in all sources of engine.
 * The prefix is set by constructor, this.engine_tag.
 *
 * @param string source name of the source
 * @param string data source data
 * @return bool true, if the signal has been handled by this slot
 */
JDBus.prototype.dataUpdated = function(source, data)
{
	// check that it's our source
	if (source.substr(0, this.engine_tag.length) != this.engine_tag)
		return false;
	// check that some data is set
	// if not then we'll think that it's caused by activateJSAPIUpdateRequests workaround
	// read more about it in engine comments
	var length = 0;
	for (var i in data)
		length++;
	if (length == 0)
		return true;

	$.debug_print("@@@ JDBus: caught dataUpdated for source '" + source + "'");

	// check for 'version' source
	if (source == this.engine_tag + "version")
	{
		// check version compatibility
		// reserved for later use
		$.message("DBus data engine has version " + data["version"] + "; it's suitable for the plasmoid");

		// ask only once
		this.engine.disconnectSource(this.engine_tag + "version", plasmoid);
		return true;
	}

	// check for 'instance name' source
	if (source == this.engine_tag + "instance name")
	{
		// this reply is received when instance name is registered or instance is renamed
		if (data.name && data.name != $.name)
		{
			$.name = data.name;
			$.message("The instance name has been changed to '" + $.name + "'");
		}
		if (!this.is_registered)
		{
			$.message("The instance name has been registered in DBus.");
			// call back
			this._registered();
		}
		else
			$.message("The instance name has been reregistered in DBus.");
		return true;
	}

	// check for 'progress factory' source
	if (source == this.engine_tag + "progress factory")
	{
		this._updateItemSet(data);
		return true;
	}

	// check for 'progress: ProgressName' source
	var progress_tag = this.engine_tag + "progress: ";
	if (source.substr(0, progress_tag.length) == progress_tag)
	{
		var name = source.substr(progress_tag.length);
		var item = $.i.findItemByName(name);
		if (!item)
		{
			$.debug_print("@@@ JDBus: can't find item with name '" + name + "'");
			return true;
		}
		this._updateItemData(item, data);
	}

	return true;
};

/**
 * Submit data to the engine using source and operation.
 *
 * Simplified interface to serviceForSource + operationDescription + startOperationCall.
 *
 * @param string service_name name of the service
 * @param string operation_name name of the operation
 * @param object data fields that are set to the operation
 * @return bool true, if service and operation were found
 */
JDBus.prototype.submit = function(service_name, operation_name, data)
{
	// try to get service
	var service = this.engine.serviceForSource(this.engine_tag + service_name);
	if (!service)
	{
		$.debug_print("@@@ JDBus: submit, '" + service_name + "' service not found");
		return false;
	}
	// try to get operation
	var operation = service.operationDescription(operation_name);
	if (!operation)
	{
		$.debug_print("@@@ JDBus: submit, '" + operation_name + "' operation not found");
		return false;
	}
	// set operation data
	for (var i in data)
		operation[i] = data[i];
	// call
	service.startOperationCall(operation);
	return true;
};

/**
 * Start registration procedures.
 *
 * Two things should happen:
 * 1. Name of the instance should be registered.
 * 2. Version of the engine should be checked for compatibility.
 *
 * @return bool true, if submit was successful
 */
JDBus.prototype.register = function()
{
	if (!this.isValid())
		return false;

	$.debug_print("@@@ JDBus: register");
	$.message("Registering the instance name '" + $.name + "' in DBus");

	// ask for version (should be a query() call, but this method doesn't work in API v1, bug #230624)
	this.engine.connectSource(this.engine_tag + "version", plasmoid);
	// turn on tracking of replies
	this.engine.connectSource(this.engine_tag + "instance name", plasmoid);
	// try to set name; result will be received in dataUpdated slot
	return this.submit("instance name", "SetName", { name: $.name });
};

/**
 * Callback for the successful registration.
 *
 * Sets up flags, emits required signals, registers existing items.
 */
JDBus.prototype._registered = function()
{
	this.is_registered = true;
	this.activated.emit();
	this._registerItems();
};

/**
 * Register a new progress in the engine.
 *
 * Submits request to the engine.
 *
 * @param JProgress item
 * @return bool true, if submit was successful
 */
JDBus.prototype.registerItem = function(item)
{
	if (!this.isActive())
		return false;

	$.debug_print("@@@ JDBus: registerItem(" + item.id + ", " + item.origin + ")");
	if (!this.accepting_items)
		$.debug_print("@@@ JDBus: registerItem, assert (this.accepting_items == true) failed");

	if (!this.submit("progress factory", "RegisterItem", { id: item.id, type: $.ProgressType.Export(item.type) }))
		return false;
	return true;
};

/**
 * Register all items of the central '$' object one by one.
 *
 * @return bool true, if all items were submitted successfully
 */
JDBus.prototype._registerItems = function()
{
	if ($.i.itemCount() == 0)
		return true;

	$.debug_print("@@@ JDBus: _registerItems");

	// add item by item
	for (var i in $.i.items)
		if (!this.registerItem($.i.items[i]))
			return false;
	return true;
};

/**
 * Update items of the central '$' object by data from 'progress factory' source.
 *
 * The following things are updated:
 * - item name
 * - new external item could be created
 * - any item could be removed by request from the engine
 *
 * See engine docs/comments for deeper explanation.
 *
 * @param object data data from 'progress factory' source
 */
JDBus.prototype._updateItemSet = function(data)
{
	$.debug_print("@@@ JDBus: _updateItemSet");

	for (var name in data)
	{
		var id = data[name].id;
		var origin = JProgress.ItemOrigin.Import(data[name].origin, JProgress.ItemOriginExternal);
		var type = $.ProgressType.Import(data[name].type, $.ProgressTypeBasic);
		var remove_request = data[name]["request: remove"];

		// try to find item
		var item = $.i.findItem(id, origin);
		var do_connect = false;
		// if it's found then just update the name
		if (item)
		{
			$.debug_print("@@@ JDBus: _updateItemSet, item " + name + " was found: #" + id + " in origin " + origin);

			// setup parameters
			if (!item.jdbus_name) // it's a first time
				$.message("A new item " + name + " (former " + item.name + ") has been added by plasmoid and registered in DBus.");
			item.name = item.jdbus_name = name;
			// remove item by request
			if (remove_request)
			{
				// turn off tracking of progress data
				this.engine.disconnectSource(this.engine_tag + "progress: " + name, plasmoid);
				// remove item from the set
				$.i.removeItem(item);
				// remove item from the engine
				this.submit("progress factory", "RemoveItem", { name: item.name });
				$.message("The item " + name + " has been removed from both plasmoid and DBus.");
				continue;
			}

			// turn on tracking of progress data
			this.engine.connectSource(this.engine_tag + "progress: " + name, plasmoid);
			// schedule connect
			do_connect = true;
		}
		// if not then this means that we probably have a new item
		else
		{
			// don't consider removed items
			if (remove_request)
				continue;
			// if it's an internal item then it's not an error cause we could delete this item and engine didn't understand it yet
			if (origin == JProgress.ItemOriginInternal)
			{
				$.debug_print("@@@ JDBus: _updateItemSet, internal item #" + id + " not found (it could be already deleted by user)");
			}
			// if it's an external item then it was created via DBus and we must create it here, in the plasmoid
			else
			{
				$.debug_print("@@@ JDBus: _updateItemSet, creating a new external item " + name + " (#" + id + ") of type " + type);

				// create a progress and setup parameters
				var progress = $.newProgress(type);
				progress.id = id;
				progress.origin = origin;
				progress.name = progress.jdbus_name = name;

				// add to the set
				$.i.addItem(progress);
				$.message("A new item " + name + " has been added via DBus.");
				// apply theme
				$.applyTheme(progress);
				// turn on tracking of progress data
				this.engine.connectSource(this.engine_tag + "progress: " + name, plasmoid);
				// schedule connect
				item = progress;
				do_connect = true;
			}
		}
		// connect item to sendItemProperty and send initial values to the engine
		// the slot generator is needed here to make prop a value, not a local variable which address js remembers
		// if we try to paste the function rigth into connect(...) then we will always get prop in that function
		// as a last item in the JDBus.progress_props array
		var slot = function(prop) { return function () { $.jdbus.sendItemProperty(this, prop); } };
		if (do_connect)
			for (var i in JDBus.progress_props)
			{
				var prop = JDBus.progress_props[i];
				if (item[prop + "Changed"] instanceof JSignal)
				{
					$.debug_print("@@@ JDBus: item " + item.name + ", " + prop + "Changed is connected with sendItemProperty");
					// connect
					item[prop + "Changed"].connect(slot(prop));
					// send initial value
					slot(prop).call(item);
				}
			}
	}
};

/**
 * Update item parameters that come from 'progress: ProgressName' source.
 *
 * If engine request some parameters to be set, then it overlaps current params.
 *
 * @param JProgress item
 * @param object data data from 'progress factory' source
 */
JDBus.prototype._updateItemData = function(item, data)
{
	$.debug_print("@@@ JDBus: _updateItemData(" + item.name + ")");

	// DBus requested plasmoid to set the value of the item
	for (var i in JDBus.progress_props)
	{
		var prop = JDBus.progress_props[i];
		if (typeof item[prop] !== "undefined" && data["request: " + prop])
		{
			// tell the engine that we received the request
			item.is_jdbus_requested = true;
			// set value (this would call sendItemProperty, if data changes)
			item[prop] = data[prop];
			// if data didn't changed, is_jdbus_requested is not cleared
			// we must call sendItemProperty manually to clean request flag
			this.sendItemProperty(item, prop);
			// clear flag anyway
			// isActive may return false, so it would not be creared by sendItemProperty
			delete item.is_jdbus_requested;
		}
	}
};

/**
 * Send a new value of the item property to the engine.
 *
 * It also checks is_jdbus_requested member of the item to send proper reply to the engine.
 *
 * @param JProgress item
 * @param string prop property name
 * @return bool true, if submit was successful
 */
JDBus.prototype.sendItemProperty = function(item, prop)
{
	$.debug_print("@@@ JDBus: sendItemProperty(" + item.name + ", " + prop + ")");

	if (!this.isActive() || typeof item[prop] === "undefined")
		return false;

	$.debug_print("@@@ JDBus: sendItemProperty, item." + prop + " = " + item[prop]);

	// tell the engine that value is changed
	// compose data
	var source = "progress: " + item.name;
	var Prop = prop.charAt(0).toUpperCase() + prop.substr(1);
	var data = {};
	data["Set" + Prop + "_is_requested"] = !!item.is_jdbus_requested; // !! is for converting to boolean
	data["Set" + Prop + "_value"] = item[prop];
	// clear flag
	delete item.is_jdbus_requested;
	// send
	return this.submit(source, "Set" + Prop, data);
};

/**
 * Schedule the item for removal in the engine.
 *
 * This is called when we want to remove the item from the plasmoid.
 *
 * @param JProgress item
 * @return bool true, if submit was successful
 */
JDBus.prototype.scheduleItemRemoval = function(item)
{
	if (!this.isActive())
		return false;

	// schedule for removal in DBus service; result will be received in dataUpdated slot
	return this.submit("progress factory", "ScheduleItemRemoval", { name: item.name });
};
