/**
 * Dialog that is based on external scripts.
 *
 * JS API v1 doesn't support many config dialogs but it's needed to setup many items.
 * So this is a workaround that launches external friendly programs to show dialogs.
 * Pros:
 * - support for more than one dialog
 * - support for custom interactive dialogs
 * - workaround for bugs in javascript bindings for KConfigXT
 *   (for ex., in QTimeEdit + DateTime kcfg type in APIv1, whatever...)
 * Cons:
 * - another external dep
 * - can't implement apply button since dialog should be closed to pass reply to us
 *
 * JDialog sends parameters to a dialog as a first argument - JSON array.
 * The dialog prints JSON reply to the output and returns zero (exit code) on success
 * or returns non-zero (exit code) on failure (or cancel).
 *
 * @license GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author Nikita Melnichenko [http://nikita.melnichenko.name]
 * @copyright Copyright 2010, Nikita Melnichenko
 */

plasmoid.include_once("exec.js"); // for scriptPath

/**
 * Constructor.
 *
 * A dialog that is called from external script.
 *
 * @param string script the name of the script relative to scripts dir
 */
function JDialog(interpreter, script)
{
	$.debug_print("@@@ JDialog: contructor");

	// script interpreter
	// this is important thing because cause zip doesn't store executable bits
	// so we can't directly execute scripts from the package
	this.interpreter = interpreter;
	// get full path of the script
	this.script = scriptPath(script);

	// define signals
	this.accepted = new JSignal(this); // dialog is accepted
	this.rejected = new JSignal(this); // dialog is rejected
}

/**
 * Display the dialog.
 *
 * @param object data dialog parameters
 * @return bool true, if script exists
 */
JDialog.prototype.show = function(data)
{
	if (!this.script)
		return false;

	var json = JSON.stringify(data);
	var command_line = commandLine(this.interpreter, [this.script, json]);
	this.receiver = command_line;
	dataEngine("executable").connectSource(command_line, plasmoid);
	return true;
};

/**
 * dataUpdated slot for events from the "executable" engine.
 *
 * JS API v1 allows to connect source to plasmoid.dataUpdated only.
 * To separate engine replies from other replies we check if source is equal to this.receiver.
 * this.receiver is set by show method.
 *
 * @param string source name of the source
 * @param string data source data
 * @return bool true, if the signal has been handled by this slot
 */
JDialog.prototype.dataUpdated = function(source, data)
{
	// check that it's our source
	if (!this.receiver || this.receiver != source)
		return false;

	$.debug_print("@@@ JDialog: caught dataUpdated for source '" + source + "'");

	// disconnect immediately
	dataEngine("executable").disconnectSource(source, plasmoid);
	delete this.receiver;

	// parse data
	var exit_code = data["exit code"];
	if (exit_code == 0)
	{
		var reply = JSON.parse(data["stdout"]);
		this.accepted.emit(reply);
	}
	else
		this.rejected.emit(exit_code);

	return true;
};
