/**
 * JLiveProgress class for progress states that automatically advanced by timer.
 *
 * @license GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author Nikita Melnichenko [http://nikita.melnichenko.name]
 * @copyright Copyright 2010, Nikita Melnichenko
 */

plasmoid.include_once("JProgress.class.js"); // JLiveProgress inherits this

/**
 * Constructor.
 *
 * A progress with timer.
 */
function JLiveProgress()
{
	// parent constructor
	JProgress.call(this);

	this.type = $.ProgressTypeLive;

	// members
	this._tick = 200;
	this.timer = new QTimer;
	this._speed = 1; // progress per second

	// connect timer
	var $this = this;
	this.timer.timeout.connect(function () { $this._advance(); });

	// define signals
	this.activeChanged = new JSignal(this); // started or stopped
	this.speedChanged = new JSignal(this); // speed is changed
}

// prototype inheritance
JLiveProgress.inheritsPrototype(JProgress);

/**
 * Property: active.
 *
 * The progress is active if and only if it's started.
 *
 * Emits "activeChanged" and "updated" signals when changed.
 */
JLiveProgress.prototype.__defineGetter__("active", function()
{
	return this.isStarted();
});
JLiveProgress.prototype.__defineSetter__("active", function(value)
{
	if (this.isStarted() != value)
	{
		$.debug_print("@@@ JLiveProgress: item " + this.name + ", active changed to " + value);
		if (value)
			this.start(); // emits "activeChanged" and "updated"
		else
			this.stop(); // emits "activeChanged" and "updated"
	}
});

/**
 * Property: speed.
 *
 * Emits "speedChanged" and "updated" signals when changed.
 */
JLiveProgress.prototype.__defineGetter__("speed", function()
{
	return this._speed;
});
JLiveProgress.prototype.__defineSetter__("speed", function(value)
{
	if (this._speed != value)
	{
		$.debug_print("@@@ JProgress: item " + this.name + ", speed changed from " + this._speed + " to " + value);
		// when we change min, fraction can't become 1, so finished never emitted
		this._speed = value;
		this.speedChanged.emit();
		this.updated.emit();
	}
});

/// Start timer.
JLiveProgress.prototype.start = function()
{
	$.debug_print("@@@ JLiveProgress: start " + this.name);
	if (this.isStarted())
		return;

	if (this.fraction() == 1)
		this.finish();
	else
	{
		// save the time of update in the object
		// this is used in _advance function
		this._update_date = new Date();
		this._setupTimerInterval();
		this.timer.start();

		this.activeChanged.emit();
		this.updated.emit();
	}
}

/// Stop timer.
JLiveProgress.prototype.stop = function()
{
	$.debug_print("@@@ JLiveProgress: stop " + this.name);
	if (!this.isStarted())
		return;

	this.timer.stop();
	if (!this.isFinished())
		this._advance();
	delete this._update_date;

	this.activeChanged.emit();
	this.updated.emit();
}

/// Finish progress, i.e. make this.fraction() == 1.
JLiveProgress.prototype.finish = function()
{
	if (this.fraction() != 1)
		this.value = this.maximum; // emits finished
	this.stop(); // could emit activeChanged
}

/**
 * Is timer started?
 *
 * @return bool true, if timer is started
 */
JLiveProgress.prototype.isStarted = function()
{
	return this.timer.active;
}

/**
 * Property: tick, the interval of update.
 */
JLiveProgress.prototype.__defineGetter__("tick", function()
{
	return this._tick;
});
JLiveProgress.prototype.__defineSetter__("tick", function(value)
{
	this._tick = value;
	if (this.isStarted())
		this._advance();
});

/**
 * Advance progress value by computing a time interval since the last update.
 *
 * We can't rely on timer interval but we can count on current time.
 * Time interval is computed as difference of current time and the time of the last update.
 * Value is advanced by speed * interval (in seconds).
 */
JLiveProgress.prototype._advance = function()
{
	// compute interval and save a new time of update in the object
	var current_date = new Date();
	var interval = current_date - this._update_date;
	this._update_date = current_date;

	// set a new value
	this.value = Math.min(this.value + this.speed * interval / 1000., this.maximum);
	// check if we need to finish or to set up new timer interval
	if (this.fraction() == 1)
		this.finish();
	else
		this._setupTimerInterval();
};

/**
 * Setup interval of internal timer.
 *
 * The problem is that we should finish at the exact time no matter what user set as a tick.
 * So if there is less time to finish than tick length then we correct timer interval.
 */
JLiveProgress.prototype._setupTimerInterval = function()
{
	var interval = Math.ceil((this.maximum - this.value) / this.speed * 1000);
	if (interval < 1)
		interval = 1;
	if (interval > this._tick)
		interval = this._tick;

	if (this.timer.interval != interval)
		this.timer.interval = interval;
};

/**
 * Export properties for configuration.
 *
 * The first letter is capital because 'export' gives syntax error (bindings issue?).
 *
 * @return object exported item
 */
JLiveProgress.prototype.Export = function()
{
	var o = JProgress.prototype.Export.call(this);
	o.type = $.ProgressType.Export(this.type);
	o.active = this.active;
	o.speed = this.speed;
	o.tick = this.tick;
	return o;
};

/**
 * Import properties after configuration.
 *
 * The first letter is capital because 'import' gives syntax error (bindings issue?).
 *
 * @param object o imported data
 */
JLiveProgress.prototype.Import = function(o)
{
	if (!o)
		return;
	this.active = importBoolean(o.active, this.active);
	this.speed = importFloat(o.speed, this.speed);
	this.tick = importInt(o.tick, this.tick);
	JProgress.prototype.Import.call(this, o);
};
