/**
 * JMultiProgress class for a set of progresses.
 *
 * Assuming all items to be objects of classes that inherit JProgress.
 *
 * @license GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author Nikita Melnichenko [http://nikita.melnichenko.name]
 * @copyright Copyright 2010, Nikita Melnichenko
 */

plasmoid.include_once("JDialog.class.js"); // used in configureItem

/**
 * Constructor.
 *
 * A set of progresses.
 */
function JMultiProgress()
{
	// all items of this group
	this.items = new Array();
	// the next unique id for new item
	this.next_id = 1;

	// define signals
	this.updated = new JSignal(this); // one of items updated or update() is called

	// define slots
	this.update = JSlot(this, this._update);
}

/**
 * Initialize a new item with default parameters.
 *
 * Should be overriden to setup progress defaults.
 *
 * @param JProgress item
 */
JMultiProgress.prototype.initItem = function(item)
{
	$.debug_print("@@@ JMultiProgress: initItem " + item.name);
	// connect updated item signal to update slot
	item.updated.connect(this.update);
};

/**
 * Add a new item to the set.
 *
 * @param JProgress progress input progress
 * @return JProgress modified item
 */
JMultiProgress.prototype.addItem = function(progress)
{
	if (!progress)
		return;

	// add item
	this.items.push(progress);
	// set unique id for internally created items
	if (progress.origin == JProgress.ItemOriginInternal)
		progress.id = this.next_id++;
	// set some name by default
	if (!progress.name)
		progress.name = "p" + progress.id;
	// set title equal to name by default
	if (!progress.title)
		progress.title = progress.name;
	// call initializer
	this.initItem(progress);
	// now we can update ui
	this.update();

	return progress;
};

/**
 * Schedule the item for removal.
 *
 * Should be overriden if some preparations need to be done before removal.
 * After all that stuff the item should be removed using removeItem method.
 *
 * @param JProgress item
 */
JMultiProgress.prototype.scheduleItemRemoval = function(item)
{
	// just remove it
	this.removeItem(item);
};

/**
 * Free all resources of the item before delete.
 *
 * Should be overriden to remove all subelements.
 *
 * @param JProgress item
 */
JMultiProgress.prototype.freeItem = function(item)
{
	$.debug_print("@@@ JMultiProgress: freeItem " + item.name);

	// stop live item if it is started
	if (typeof item.stop === "function")
		item.stop();

	// disconnect updated item signal
	item.updated.disconnect(this.update);
};

/**
 * Remove the item from the set.
 *
 * @param JProgress item
 */
JMultiProgress.prototype.removeItem = function(item)
{
	// find and delete the item
	for (var i in this.items)
		if (item === this.items[i])
		{
			this.items.splice(i, 1);
			this.freeItem(item);
			break;
		}
	// update ui
	this.update();
};

/**
 * Export configuration properties of the item.
 *
 * Should be overriden to add support for configuration of ui props.
 *
 * @param JProgress item
 * @return object exported item
 */
JMultiProgress.prototype.exportItem = function(item)
{
	var o = item.Export();
	// override and add ui props here
	return o;
};

/**
 * Import configuration properties of the item.
 *
 * Should be overriden to add support for configuration of ui props.
 *
 * @param JProgress item
 * @param object o imported data
 */
JMultiProgress.prototype.importItem = function(item, o)
{
	item.Import(o);
	// override and add ui props here
};

/**
 * Export all items.
 *
 * @return object exported items
 */
JMultiProgress.prototype.exportItems = function()
{
	var o = [];
	for (var i in this.items)
		o[i] = this.exportItem(this.items[i]);
	return o;
};

/**
 * Import all items.
 *
 * It's supposed that number of items before import = 0.
 *
 * @param object o imported data
 */
JMultiProgress.prototype.importItems = function(o)
{
	if (!o)
		return;

	// replace default action so that new items would not report on value change
	// their actions will be imported from o object
	var default_action = $.defaults.actionOnFinish;
	$.defaults.actionOnFinish = new JAction();

	// import item by item
	for (var i in o)
	{
		var type = $.ProgressType.Import(o[i].type, $.ProgressTypeBasic);
		var item = this.addItem($.newProgress(type));
		this.importItem(item, o[i]);
	}

	// restore default action
	$.defaults.actionOnFinish = default_action;
};

/**
 * Find item by identifier and origin.
 *
 * @param int id item identifier
 * @param JProgress.ItemOrigin origin item origin
 * @return JProgress item that has given id and origin (or null)
 */
JMultiProgress.prototype.findItem = function(id, origin)
{
	for (var i in this.items)
	{
		var item = this.items[i];
		if (item.id == id && item.origin == origin)
			return item;
	}
	return null;
};

/**
 * Find item by name.
 *
 * @param string name item name
 * @return JProgress item that has given name
 */
JMultiProgress.prototype.findItemByName = function(name)
{
	for (var i in this.items)
	{
		var item = this.items[i];
		if (item.name == name)
			return item;
	}
	return null;
};

/**
 * Get number of items.
 *
 * @return int item count
 */
JMultiProgress.prototype.itemCount = function()
{
	return this.items.length;
};

/**
 * Force update of the multiprogress.
 */
JMultiProgress.prototype._update = function()
{
	this.updated.emit();
};

/**
 * Configure the item.
 *
 * Create a dialog with current progress parameters.
 *
 * @param JProgress item
 */
JMultiProgress.prototype.configureItem = function(item)
{
	$.debug_print("@@@ JMultiProgress: configureItem " + item.name);
	if (this.dialog)
	{
		$.debug_print("@@@ JMultiProgress: configureItem, dialog already shown");
		return;
	}

	this.dialog = new JDialog("python", "dialog-config-progress.py");
	this.dialog.delete_code = 100; // used in rejected handler
	var $this = this;
	this.dialog.accepted.connect(function (data)
	{
		// handle
		$this.itemConfigured(item, data);
		// clean up
		$this.dialogClosed();
	});
	this.dialog.rejected.connect(function (code)
	{
		// delete item if requested
		if (code == this.delete_code)
			$this.scheduleItemRemoval(item);
		// clean up
		$this.dialogClosed();
	});
	this.dialog.show(this.exportItem(item));
};

/**
 * Callback for successful configuration.
 *
 * Process data from the dialog.
 *
 * @param JProgress item
 * @param object data
 */
JMultiProgress.prototype.itemConfigured = function(item, data)
{
	if ($.debug)
	{
		print("@@@ JMultiProgress: itemConfigured " + item.name + ", data received:");
		var_dump(data);
	}
	this.importItem(item, data);
};

/// Callback for closed dialog.
JMultiProgress.prototype.dialogClosed = function()
{
	$.debug_print("@@@ JMultiProgress: dialogClosed");
	delete this.dialog;
};

/**
 * Universal start/stop/reset action.
 *
 * When progress is started, we stop it.
 * When progress is finished, we reset it.
 * When progress is stopped, we start it.
 */
JMultiProgress.prototype.doStartStopResetItem = function(item)
{
	$.debug_print("@@@ JMultiProgress: doStartStopReset " + item.name);

	if (item.isStarted()) // started -> stopped
		item.stop();
	else if (item.isFinished()) // finished -> reset
		item.reset();
	else // stopped -> started
		item.start();
};
