/**
 * JPollProgress class for progress states that automatically advanced by timer.
 *
 * @license GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author Nikita Melnichenko [http://nikita.melnichenko.name]
 * @copyright Copyright 2010, Nikita Melnichenko
 */

plasmoid.include_once("JProgress.class.js"); // JPollProgress inherits this

/**
 * Constructor.
 *
 * A progress with timer.
 */
function JPollProgress()
{
	// parent constructor
	JProgress.call(this);

	this.type = $.ProgressTypePoll;

	// members
	this.timer = new QTimer;
	this.tick = 2000;
	this.command_line = "";
	this.wait_for_answer = false; // flag: we wait async answer from command

	// connect timer
	var $this = this;
	this.timer.timeout.connect(function () { $this.poll(); });

	// define signals
	this.activeChanged = new JSignal(this); // started or stopped
}

// prototype inheritance
JPollProgress.inheritsPrototype(JProgress);

/**
 * Property: active.
 *
 * The progress is active if and only if it's started.
 *
 * Emits "activeChanged" and "updated" signals when changed.
 */
JPollProgress.prototype.__defineGetter__("active", function()
{
	return this.timer.active;
});
JPollProgress.prototype.__defineSetter__("active", function(value)
{
	if (this.timer.active != value)
	{
		$.debug_print("@@@ JPollProgress: item " + this.name + ", active changed to " + value);
		if (value)
		{
			this.timer.start();
			this.poll();
		}
		else
			this.timer.stop();
		this.activeChanged.emit();
		this.updated.emit();
	}
});

/**
 * Property: tick, the interval of update.
 */
JPollProgress.prototype.__defineGetter__("tick", function()
{
	return this.timer.interval;
});
JPollProgress.prototype.__defineSetter__("tick", function(value)
{
	this.timer.interval = value;
});

/// poll a command for a new parameters
JPollProgress.prototype.poll = function()
{
	$.debug_print("@@@ JPollProgress: item " + this.name + ", poll, currently waits = " + this.wait_for_answer);
	if (!this.command_line || this.wait_for_answer)
		return;

	// save the current command line in the case user change it
	this.receiver = this.command_line;
	this.wait_for_answer = true;
	$.debug_print("@@@ JPollProgress: item " + this.name + ", poll, executing command: " + this.command_line);
	dataEngine("executable").connectSource(this.command_line, plasmoid);
};

/**
 * dataUpdated slot for events from the "executable" engine.
 *
 * JS API v1 allows to connect source to plasmoid.dataUpdated only.
 * To separate engine replies from other replies we check if source is equal to this.receiver.
 * this.receiver is set by poll method.
 *
 * @param string source name of the source
 * @param string data source data
 * @return bool true, if the signal has been handled by this slot
 */
JPollProgress.prototype.dataUpdated = function(source, data)
{
	// check that it's our source
	if (!this.receiver || this.receiver != source)
		return false;

	$.debug_print("@@@ JPollProgress: item " + this.name + " caught dataUpdated for source '" + source + "'");

	// disconnect immediately
	dataEngine("executable").disconnectSource(source, plasmoid);
	delete this.receiver;
	this.wait_for_answer = false;

	// parse data
	var exit_code = data["exit code"];
	$.debug_print("@@@ JPollProgress: item " + this.name + ", command exit code is " + exit_code);
	if (exit_code == 0)
		this.handle(data["stdout"]);

	return true;
};

/**
 * Handle incoming data, the command output.
 *
 * Try to parse JSON or consider output as a progress value.
 *
 * @param string data command output
 */
JPollProgress.prototype.handle = function(data)
{
	var o;
	try
	{
		o = JSON.parse(data);
	}
	catch(error)
	{
		$.debug_print("@@@ JPollProgress: item " + this.name + ", handle, failed to parse JSON data");
		o = 0;
	}

	if (typeof o == "number")
		o = { value: o };
	JProgress.prototype.Import.call(this, o);
};

/**
 * Export properties for configuration.
 *
 * The first letter is capital because 'export' gives syntax error (bindings issue?).
 *
 * @return object exported item
 */
JPollProgress.prototype.Export = function()
{
	var o = JProgress.prototype.Export.call(this);
	o.type = $.ProgressType.Export(this.type);
	o.active = this.active;
	o.tick = this.tick;
	o.command_line = this.command_line;
	return o;
};

/**
 * Import properties after configuration.
 *
 * The first letter is capital because 'import' gives syntax error (bindings issue?).
 *
 * @param object o imported data
 */
JPollProgress.prototype.Import = function(o)
{
	if (!o)
		return;
	this.active = importBoolean(o.active, this.active);
	this.tick = importInt(o.tick, this.tick);
	this.command_line = importString(o.command_line, this.command_line);
	JProgress.prototype.Import.call(this, o);
};
