/**
 * JProgress class for general progress state.
 *
 * @license GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author Nikita Melnichenko [http://nikita.melnichenko.name]
 * @copyright Copyright 2010, Nikita Melnichenko
 */

plasmoid.include_once("JAction.class.js"); // JProgress actionOnFinish uses JAction

/**
 * Constructor.
 *
 * A progress.
 */
function JProgress()
{
	this.type = $.ProgressTypeBasic;

	this._minimum = 0;
	this._maximum = 100;
	this._value = 0;

	this.id = 0; // identifier that should be unique within origin
	this.origin = JProgress.ItemOriginInternal; // origin of the progress
	this.name = ""; // name that mainly used for messages and in dbus
	this._title = ""; // title that use in notifications and on plasmoid

	// define signals
	this.updated = new JSignal(this); // some props were updated
	this.minimumChanged = new JSignal(this); // minimum is updated
	this.maximumChanged = new JSignal(this); // maximum is updated
	this.valueChanged = new JSignal(this); // value is updated
	this.titleChanged = new JSignal(this); // title is updated
	this.finished = new JSignal(this); // this.fraction() == 1 after advance

	// action executed on finish
	if ($.defaults.actionOnFinish)
		this.actionOnFinish = clone($.defaults.actionOnFinish);
	else
		this.actionOnFinish = new JAction();
	this.finished.connect(function () { this.actionOnFinish.execute(this); });
}

// define ItemOrigin enum
JProgress.Enum("ItemOrigin",
{
	"Internal": 0,
	"External": 1
});

/**
 * Get progress as a fraction (number in [0, 1]).
 *
 * @return float progress fraction
 */
JProgress.prototype.fraction = function()
{
	var eps = 1e-6;
	if (this.minimum + eps > this.maximum || this._value < this.minimum + eps)
		return 0;
	if (this._value > this.maximum - eps)
		return 1;
	return (this._value - this.minimum) / (this.maximum - this.minimum);
};

/**
 * Property: minimum.
 *
 * Emits "minimumChanged" and "updated" signals when changed.
 */
JProgress.prototype.__defineGetter__("minimum", function()
{
	return this._minimum;
});
JProgress.prototype.__defineSetter__("minimum", function(value)
{
	if (this._minimum != value)
	{
		$.debug_print("@@@ JProgress: item " + this.name + ", minimum changed from " + this._minimum + " to " + value);
		// when we change min, fraction can't become 1, so finished never emitted
		this._minimum = value;
		this.minimumChanged.emit();
		this.updated.emit();
	}
});

/**
 * Property: maximum.
 *
 * Emits "maximumChanged" and "updated" signals when changed.
 * Emits "finished" when fraction becomes == 1.
 */
JProgress.prototype.__defineGetter__("maximum", function()
{
	return this._maximum;
});
JProgress.prototype.__defineSetter__("maximum", function(value)
{
	if (this._maximum != value)
	{
		$.debug_print("@@@ JProgress: item " + this.name + ", maximum changed from " + this._maximum + " to " + value);
		var fraction_before = this.fraction();
		this._maximum = value;
		var fraction_after = this.fraction();
		if (fraction_before < 1 && fraction_after == 1)
			this.finished.emit();
		this.maximumChanged.emit();
		this.updated.emit();
	}
});

/**
 * Property: value.
 *
 * Emits "valueChanged" and "updated" signals when changed.
 * Emits "finished" when fraction becomes == 1.
 */
JProgress.prototype.__defineGetter__("value", function()
{
	return this._value;
});
JProgress.prototype.__defineSetter__("value", function(value)
{
	if (this._value != value)
	{
		$.debug_print("@@@ JProgress: item " + this.name + ", value changed from " + this._value + " to " + value);
		var fraction_before = this.fraction();
		this._value = value;
		var fraction_after = this.fraction();
		if (fraction_before < 1 && fraction_after == 1)
			this.finished.emit();
		this.valueChanged.emit();
		this.updated.emit();
	}
});

/**
 * Property: title.
 *
 * Emits "titleChanged" and "updated" signals when changed.
 */
JProgress.prototype.__defineGetter__("title", function()
{
	return this._title;
});
JProgress.prototype.__defineSetter__("title", function(value)
{
	if (this._title != value)
	{
		$.debug_print("@@@ JProgress: item " + this.name + ", title changed from '" + this._title + "' to '" + value + "'");
		// when we change min, fraction can't become 1, so finished never emitted
		this._title = value;
		this.titleChanged.emit();
		this.updated.emit();
	}
});

/**
 * Is timer finished?
 *
 * @return bool true, if progress is done (fraction = 1).
 */
JProgress.prototype.isFinished = function()
{
	return this.fraction() == 1;
}

/// Reset value, i.e. set to minimum.
JProgress.prototype.reset = function()
{
	this.value = this.minimum;
};

/**
 * Export properties for configuration.
 *
 * The first letter is capital because 'export' gives syntax error (bindings issue?).
 *
 * @return object exported item
 */
JProgress.prototype.Export = function()
{
	var o = {};
	o.type = $.ProgressType.Export(this.type);
	o.minimum = this.minimum;
	o.maximum = this.maximum;
	o.value = this.value;
	o.name = this.name;
	o.title = this.title;
	o.actionOnFinish = this.actionOnFinish.Export();
	return o;
};

/**
 * Import properties after configuration.
 *
 * The first letter is capital because 'import' gives syntax error (bindings issue?).
 *
 * @param object o imported data
 */
JProgress.prototype.Import = function(o)
{
	if (!o)
		return;
	this.minimum = importFloat(o.minimum, this.minimum);
	this.maximum = importFloat(o.maximum, this.maximum);
	this.value = importFloat(o.value, this.value);
	this.name = importString(o.name, this.name);
	this.title = importString(o.title, this.title);
	this.actionOnFinish.Import(o.actionOnFinish);
};
