/**
 * Buttons for a single progress on multiprogress widget.
 *
 * @license GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author Nikita Melnichenko [http://nikita.melnichenko.name]
 * @copyright Copyright 2010, Nikita Melnichenko
 */

/**
 * Constructor.
 *
 * A button container and handler.
 *
 * @param JMultiProgress multiprogress
 * @param string button_theme name of a theme
 * @param int max_button_size maximum button size
 */
function JProgressButtons(multiprogress, button_theme, max_button_size)
{
	this.widget = multiprogress;
	this.is_config_button_visible = true;
	this.is_play_button_visible = true;
	this.max_button_size = max_button_size;
	this.svg_config_button = plasmoid.file("images", button_theme + "-config-button.svg");
	this.svg_play_button = plasmoid.file("images", button_theme + "-play-button.svg");
	this.svg_pause_button = plasmoid.file("images", button_theme + "-pause-button.svg");
	this.svg_reset_button = plasmoid.file("images", button_theme + "-reload-button.svg");
}

/**
 * Initialize a new item with default parameters.
 *
 * @param JProgress item
 */
JProgressButtons.prototype.initItem = function(item)
{
	var ui = item.ui;
	var $this = this;
	var $widget = this.widget;

	// config button
	ui.iw_config_button = new IconWidget;
	ui.iw_config_button.svg = this.svg_config_button;
	ui.iw_config_button.minimumSize = new QSizeF(3, 3);
	ui.iw_config_button.maximumSize = new QSizeF(this.max_button_size, this.max_button_size);
	ui.iw_config_button.opacity = 0.8;
	ui.iw_config_button.visible = this.is_config_button_visible;
	ui.iw_config_button.clicked.connect(function ()
	{
		$widget.configureItem(item);
	});

	// play/pause/reset button
	var is_live_progress = typeof item.isStarted === "function";
	if (is_live_progress)
	{
		ui.iw_play_button = new IconWidget;
		ui.iw_play_button.minimumSize = new QSizeF(3, 3);
		ui.iw_play_button.maximumSize = new QSizeF(this.max_button_size, this.max_button_size);
		ui.iw_play_button.opacity = 0.8;
		ui.iw_play_button.visible = this.is_play_button_visible;
		ui.iw_play_button.clicked.connect(function ()
		{
			$widget.doStartStopResetItem(item);
		});
		// slot for updating button
		ui.play_button_update = JSlot(item, function () { $this.updateStartStopResetButton(this); });
		// update button on parameter change events, for ex., from config dialog, or on start/stop events
		item.updated.connect(ui.play_button_update);
		// and update right now
		ui.play_button_update();
	}
}

/**
 * Free all resources of the item before delete.
 *
 * @param JProgress item
 */
JProgressButtons.prototype.freeItem = function(item)
{
	// remove buttons
	var ui = item.ui;
	if (ui.iw_config_button)
		ui.iw_config_button.deleteLater();
	if (ui.iw_play_button)
	{
		item.updated.disconnect(item.ui.play_button_update);
		delete item.ui.play_button_update;
		ui.iw_play_button.deleteLater();
	}
};

/**
 * Set proper image of universal start/stop/reset button depending the item.
 *
 * @param JLiveProgress item
 */
JProgressButtons.prototype.updateStartStopResetButton = function(item)
{
	if (item.isStarted()) // started: display pause
		item.ui.iw_play_button.svg = this.svg_pause_button;
	else if (item.isFinished()) // finished: display reset
		item.ui.iw_play_button.svg = this.svg_reset_button;
	else // stopped: display start
		item.ui.iw_play_button.svg = this.svg_play_button;
};

/**
 * Show or hide config button.
 *
 * @param bool value visibility
 */
JProgressButtons.prototype.setConfigButtonsVisibility = function(value)
{
	this.is_config_button_visible = value;
	var n = this.widget.items.length;
	if (n == 0)
		return;

	for (var i = 0; i < n; i++)
	{
		var button = this.widget.items[i].ui.iw_config_button;
		button.visible = value;
	}
};

/**
 * Show or hide play button.
 *
 * @param bool value visibility
 */
JProgressButtons.prototype.setPlayButtonsVisibility = function(value)
{
	this.is_play_button_visible = value;
	var n = this.widget.items.length;
	if (n == 0)
		return;

	for (var i = 0; i < n; i++)
	{
		var button = this.widget.items[i].ui.iw_play_button;
		if (!button)
			continue;
		button.visible = value;
	}
};
