/**
 * JUIProgressBar class.
 *
 * These UI elements look like colored bars.
 *
 * @license GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author Nikita Melnichenko [http://nikita.melnichenko.name]
 * @copyright Copyright 2010, Nikita Melnichenko
 */

plasmoid.include_once("JMultiProgress.class.js"); // JUIProgressBar inherits this
plasmoid.include_once("JProgressButtons.class.js"); // this.buttons

/**
 * Constructor.
 *
 * A multiprogress interface that shows progress bars.
 */
function JUIProgressBar()
{
	// parent constructor
	JMultiProgress.call(this);

	this.type = $.UITypeBars;

	// init props
	this.rect = new QRectF();
	this.max_bar_rel_width = 1./3;  // maximum bar width as a part of this.rect.height

	// style that is used when no items are in here
	this.no_items_style = {};
	this.no_items_style.border_color = new QColor("darkgrey");
	this.no_items_style.border_width = 2;
	this.no_items_style.center_rect_size = 3;

	// buttons
	this.buttons = new JProgressButtons(this, "circle", 48);
}

// prototype inheritance
JUIProgressBar.inheritsPrototype(JMultiProgress);

/**
 * Initialize a new item with default parameters.
 *
 * @param JProgress item
 */
JUIProgressBar.prototype.initItem = function(item)
{
	JMultiProgress.prototype.initItem.call(this, item);

	// style goes to ui member
	var ui = item.ui = {};
	ui.color = new QColor("white");
	ui.border_color = new QColor("black");
	ui.border_width = 2;
	ui.background_border_color = new QColor("darkgrey");
	ui.background_border_width = 1;

	// create buttons
	this.buttons.initItem(item);
}

/**
 * Free all resources of the item before delete.
 *
 * @param JProgress item
 */
JUIProgressBar.prototype.freeItem = function(item)
{
	// remove buttons
	this.buttons.freeItem(item);

	JMultiProgress.prototype.freeItem.call(this, item);
};

/**
 * Export configuration properties of the item.
 *
 * @param JProgress item
 * @return object exported item
 */
JUIProgressBar.prototype.exportItem = function(item)
{
	var o = item.Export();
	var ui = item.ui;
	o.ui = {};
	o.ui.type = $.UIType.Export(this.type);
	o.ui.color = QColor.Export(ui.color);
	o.ui.border_color = QColor.Export(ui.border_color);
	o.ui.background_border_color = QColor.Export(ui.background_border_color);
	return o;
};

/**
 * Import configuration properties of the item.
 *
 * @param JProgress item
 * @param object o imported data
 */
JUIProgressBar.prototype.importItem = function(item, o)
{
	if (o && o.ui)
	{
		var ui = item.ui;
		ui.color = QColor.Import(o.ui.color, ui.color);
		ui.border_color = QColor.Import(o.ui.border_color, ui.border_color);
		ui.background_border_color = QColor.Import(o.ui.background_border_color, ui.background_border_color);
		item.updated.emit()
	}

	item.Import(o); // this will call update
};

/**
 * Tell widget to draw only within the given rectangle.
 *
 * @param QRectF rect rectangle for drawing widget in
 */
JUIProgressBar.prototype.fitInRect = function(rect)
{
	this.rect = rect.clone();
};

/**
 * Paint a single progress.
 *
 * Draw nothing (fraction == 0) or a bar (fraction > 0)
 *
 * @param QPainter painter
 * @param int i index of the item
 * @param QRectF rect rectangle for a bar
 */
JUIProgressBar.prototype.paintItemProgress = function(painter, i, rect)
{
	var progress = this.items[i];

	// setup painter
	var pen = new QPen;
	pen.color = progress.ui.border_color;
	pen.width = progress.ui.border_width;
	painter.pen = pen;
	painter.setBrushColor(progress.ui.color);

	// draw progress
	var fraction = progress.fraction();
	// draw a bar (fraction > 0)
	if (fraction != 0)
	{
		// handle border width
		var prect = rect.adjustedUniformly(-pen.width / 2);
		// compute destination rectangle for current fraction
		prect.adjust(0, Math.round(prect.height * (1 - fraction)), 0, 0);
		painter.drawRect(prect);
	}
};

/**
 * Paint background of a single progress.
 *
 * @param QPainter painter
 * @param int i index of the item
 * @param QRectF rect rectangle for a bar
 */
JUIProgressBar.prototype.paintItemBackground = function(painter, i, rect)
{
	var progress = this.items[i];

	// setup painter
	var pen = new QPen;
	pen.color = progress.ui.background_border_color;
	pen.width = progress.ui.background_border_width;
	painter.pen = pen;
	painter.setBrushColor(new QColor("transparent"));

	// handle border width
	var prect = rect.adjustedUniformly(-pen.width / 2);
	// draw background
	painter.drawRect(prect);
};

/**
 * Paint the widget when no items are exist.
 *
 * Draw a circle without any fill and a center point.
 *
 * @param QPainter painter
 */
JUIProgressBar.prototype.paintNoItems = function(painter)
{
	// setup painter
	var pen = new QPen;
	pen.color = this.no_items_style.border_color;
	pen.width = this.no_items_style.border_width;
	painter.pen = pen;
	painter.setBrushColor(new QColor("transparent"));

	// draw bounding rectangle
	var prect = this.rect.adjustedUniformly(-pen.width / 2);
	painter.drawRect(prect);

	// setup painter
	painter.setBrushColor(this.no_items_style.border_color);

	// draw center rectangle
	var center = this.rect.center();
	var size = this.no_items_style.center_rect_size;
	painter.drawRect(center.x - size / 2, center.y - size / 2, size, size);
};

/**
 * Paint the widget.
 *
 * Draw items and their backgrounds or call paintNoItems if there is no items.
 *
 * @param QPainter painter
 */
JUIProgressBar.prototype.paint = function(painter)
{
	var n = this.items.length;
	if (n == 0)
	{
		this.paintNoItems(painter);
		return;
	}

	// width of a single bar
	var bar_width = Math.min(this.rect.width / n, this.max_bar_rel_width * this.rect.height);
	// the most left point of the left bar
	var x_start = Math.round(this.rect.center().x - bar_width * n / 2);

	// set fine border width
	for (var i = 0; i < n; i++)
		this.items[i].ui.border_width = bar_width < 10 ? 1 : 2;

	for (var i = 0; i < n; i++)
	{
		// draw item
		var item_rect = new QRectF(x_start + i * bar_width, this.rect.y, bar_width, this.rect.height);
		this.paintItemBackground(painter, i, item_rect);
		this.paintItemProgress(painter, i, item_rect);
		// move buttons to their positions and properly resize them
		this.setupButtons(i, item_rect);
	}
};

/**
 * Arrange buttons for the item.
 *
 * Move buttons to their positions and properly resize them.
 *
 * @param int i index of the item
 * @param QRectF rect rectangle for a bar
 */
JUIProgressBar.prototype.setupButtons = function(i, rect)
{
	var center = rect.center();
	var size = Math.min(rect.width, rect.height / 2);
	size = Math.min(size, this.buttons.max_button_size);

	var button = this.items[i].ui.iw_config_button;
	button.geometry = new QRectF(center.x - size / 2, center.y, size, size);

	button = this.items[i].ui.iw_play_button;
	if (button)
		button.geometry = new QRectF(center.x - size / 2, center.y - size, size, size);
};
