/**
 * JUIProgressCircle class.
 *
 * These UI elements look like colored pies of circle.
 *
 * @license GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author Nikita Melnichenko [http://nikita.melnichenko.name]
 * @copyright Copyright 2010, Nikita Melnichenko
 */

plasmoid.include_once("JMultiProgress.class.js"); // JUIProgressCircle inherits this
plasmoid.include_once("JProgressButtons.class.js"); // this.buttons

/**
 * Constructor.
 *
 * A multiprogress interface that shows circle progresses.
 */
function JUIProgressCircle()
{
	// parent constructor
	JMultiProgress.call(this);

	this.type = $.UITypeCircles;

	// init props
	this.center = new QPoint(0, 0);
	this.radius = 0;

	// style that is used when no items are in here
	this.no_items_style = {};
	this.no_items_style.border_color = new QColor("darkgrey");
	this.no_items_style.border_width = 2;
	this.no_items_style.center_point_radius = 3;

	// buttons
	this.buttons = new JProgressButtons(this, "circle", 48);
}

// prototype inheritance
JUIProgressCircle.inheritsPrototype(JMultiProgress);

// define Direction enum
JUIProgressCircle.Enum("Direction",
{
	"Clockwise": 0,
	"Counterclockwise": 1
});

// define StartingPoint enum
JUIProgressCircle.Enum("StartingPoint",
{
	"Top": 0,
	"Right": 3,
	"Bottom": 2,
	"Left": 1
});

// define DisplayedPart enum
JUIProgressCircle.Enum("DisplayedPart",
{
	"Elapsed": 0,
	"Remaining": 1
});

/**
 * Initialize a new item with default parameters.
 *
 * @param JProgress item
 */
JUIProgressCircle.prototype.initItem = function(item)
{
	JMultiProgress.prototype.initItem.call(this, item);

	// style goes to ui member
	var ui = item.ui = {};
	ui.color = new QColor("white");
	ui.border_color = new QColor("black");
	ui.border_width = 2;
	ui.background_color = new QColor("lightgrey");
	ui.background_border_color = new QColor("darkgrey");
	ui.background_border_width = 1;
	ui.direction = JUIProgressCircle.DirectionClockwise;
	ui.starting_point = JUIProgressCircle.StartingPointTop;
	ui.displayed_part = JUIProgressCircle.DisplayedPartElapsed;

	// create buttons
	this.buttons.initItem(item);
}

/**
 * Free all resources of the item before delete.
 *
 * @param JProgress item
 */
JUIProgressCircle.prototype.freeItem = function(item)
{
	// remove buttons
	this.buttons.freeItem(item);

	JMultiProgress.prototype.freeItem.call(this, item);
};

/**
 * Export configuration properties of the item.
 *
 * @param JProgress item
 * @return object exported item
 */
JUIProgressCircle.prototype.exportItem = function(item)
{
	var o = item.Export();
	var ui = item.ui;
	o.ui = {};
	o.ui.type = $.UIType.Export(this.type);
	o.ui.color = QColor.Export(ui.color);
	o.ui.border_color = QColor.Export(ui.border_color);
	o.ui.background_color = QColor.Export(ui.background_color);
	o.ui.background_border_color = QColor.Export(ui.background_border_color);
	o.ui.direction = JUIProgressCircle.Direction.Export(ui.direction);
	o.ui.starting_point = JUIProgressCircle.StartingPoint.Export(ui.starting_point);
	o.ui.displayed_part = JUIProgressCircle.DisplayedPart.Export(ui.displayed_part);
	return o;
};

/**
 * Import configuration properties of the item.
 *
 * @param JProgress item
 * @param object o imported data
 */
JUIProgressCircle.prototype.importItem = function(item, o)
{
	if (o && o.ui)
	{
		var ui = item.ui;
		ui.color = QColor.Import(o.ui.color, ui.color);
		ui.border_color = QColor.Import(o.ui.border_color, ui.border_color);
		ui.background_color = QColor.Import(o.ui.background_color, ui.background_color);
		ui.background_border_color = QColor.Import(o.ui.background_border_color, ui.background_border_color);
		ui.direction = JUIProgressCircle.Direction.Import(o.ui.direction, ui.direction);
		ui.starting_point = JUIProgressCircle.StartingPoint.Import(o.ui.starting_point, ui.starting_point);
		ui.displayed_part = JUIProgressCircle.DisplayedPart.Import(o.ui.displayed_part, ui.displayed_part);
		item.updated.emit()
	}

	item.Import(o); // this will call update
};

/**
 * Tell widget to draw only within the given rectangle.
 *
 * @param QRectF rect rectangle for drawing widget in
 */
JUIProgressCircle.prototype.fitInRect = function(rect)
{
	this.center = rect.center();
	this.radius = Math.min(rect.width, rect.height) / 2;
};

/**
 * Compute minimal rectangle that contains the circle.
 *
 * @param QPoint center center of the circle
 * @param float radius radius of the circle
 * @return QRectF minimal rectangle that contains the circle
 */
JUIProgressCircle.prototype.computeRectForCircle = function(center, radius)
{
	return new QRectF(center.x - radius, center.y - radius, 2 * radius, 2 * radius);
};

/**
 * Paint a single progress.
 *
 * Draw nothing (fraction == 0) or a pie (0 < fraction < 1) or a full circle (fraction == 1)
 *
 * @param QPainter painter
 * @param int i index of the item
 * @param float radius radius of the circle
 * @param bool border_only draw only a border of a pie
 */
JUIProgressCircle.prototype.paintItemProgress = function(painter, i, radius, border_only)
{
	var progress = this.items[i];

	// setup painter
	var pen = new QPen;
	pen.color = progress.ui.border_color;
	pen.width = progress.ui.border_width;
	painter.pen = pen;
	if (border_only)
		painter.setBrushColor(new QColor("transparent")); // used if we draw full ellipse
	else
		painter.setBrushColor(progress.ui.color);

	// draw progress
	var rect = this.computeRectForCircle(this.center, radius);
	var fraction = progress.fraction();
	// apply displayed part on fraction value
	if (progress.ui.displayed_part == JUIProgressCircle.DisplayedPartRemaining)
		fraction = 1 - fraction;
	// draw a full circle (fraction == 1)
	if (fraction == 1)
		painter.drawEllipse(rect);
	// draw a pie (0 < fraction < 1)
	else if (fraction != 0)
	{
		// here is the place where exact values of Direction and StartingPoint enums are used
		var start = (progress.ui.starting_point + 1) / 4;
		var span = (-1 + 2 * progress.ui.direction) * fraction;

		var pie_units = 360 * 16;
		if (border_only)
			painter.drawArc(rect, start * pie_units, span * pie_units);
		else
			painter.drawPie(rect, start * pie_units, span * pie_units);
	}
};

/**
 * Paint background of a single progress.
 *
 * @param QPainter painter
 * @param int i index of the item
 * @param float radius radius of the circle
 */
JUIProgressCircle.prototype.paintItemBackground = function(painter, i, radius)
{
	var progress = this.items[i];

	// setup painter
	var pen = new QPen;
	pen.color = progress.ui.background_border_color;
	pen.width = progress.ui.background_border_width;
	painter.pen = pen;
	painter.setBrushColor(progress.ui.background_color);

	// draw background
	var rect = this.computeRectForCircle(this.center, radius);
	painter.drawEllipse(rect);
};

/**
 * Paint the widget when no items are exist.
 *
 * Draw a circle without any fill and a center point.
 *
 * @param QPainter painter
 */
JUIProgressCircle.prototype.paintNoItems = function(painter)
{
	// setup painter
	var pen = new QPen;
	pen.color = this.no_items_style.border_color;
	pen.width = this.no_items_style.border_width;
	painter.pen = pen;
	painter.setBrushColor(new QColor("transparent"));

	// draw big circle
	// R helps not to draw outside of the specified rectangle
	var R = this.radius - pen.width / 2;
	var rect = this.computeRectForCircle(this.center, R);
	painter.drawEllipse(rect);

	// setup painter
	painter.setBrushColor(this.no_items_style.border_color);

	// draw center point
	var rect = this.computeRectForCircle(this.center, this.no_items_style.center_point_radius);
	painter.drawEllipse(rect);
};

/**
 * Paint the widget.
 *
 * Draw items and their backgrounds or call paintNoItems if there is no items.
 *
 * @param QPainter painter
 */
JUIProgressCircle.prototype.paint = function(painter)
{
	var n = this.items.length;
	if (n == 0)
	{
		this.paintNoItems(painter);
		return;
	}

	// set fine border width
	for (var i = 0; i < n; i++)
		this.items[i].ui.border_width = this.radius / n < 10 ? 1 : 2;
	// R helps not to draw outside of the specified rectangle
	var R = this.radius - Math.max(this.items[n - 1].ui.border_width, this.items[n - 1].ui.background_border_width) / 2;

	// draw circles
	// the order of drawing is important since background erases lines of pies
	this.paintItemBackground(painter, n - 1, R);
	for (var i = n - 1; i > 0; i--)
	{
		this.paintItemProgress(painter, i, R * (i + 1) / n, false);
		this.paintItemBackground(painter, i - 1, R * i / n);
		this.paintItemProgress(painter, i, R * i / n, true);
	}
	this.paintItemProgress(painter, i, R * (i + 1) / n, false);

	// move buttons to their positions and properly resize them
	this.setupButtons();
};

/**
 * Arrange buttons.
 *
 * Move buttons to their positions and properly resize them.
 */
JUIProgressCircle.prototype.setupButtons = function()
{
	var n = this.items.length;
	if (n == 0)
		return;

	// radius of the widget
	var R = this.radius;
	// radius of a button
	var r = Math.min(R / (2 * n), this.buttons.max_button_size / 2);

	// setup configure buttons
	for (var i = 0; i < n; i++)
	{
		var button = this.items[i].ui.iw_config_button;
		var center = new QPoint(this.center.x + (2 * i + 1) * R / (2 * n), this.center.y);
		button.geometry = this.computeRectForCircle(center, r);
	}

	// setup play/pause buttons
	for (var i = 0; i < n; i++)
	{
		var button = this.items[i].ui.iw_play_button;
		if (!button)
			continue;
		var center = new QPoint(this.center.x - (2 * i + 1) * R / (2 * n), this.center.y);
		button.geometry = this.computeRectForCircle(center, r);
	}
};
