/**
 * JUIProgressMeter class.
 *
 * Plasma meters with buttons and labels.
 *
 * @license GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author Nikita Melnichenko [http://nikita.melnichenko.name]
 * @copyright Copyright 2010, Nikita Melnichenko
 */

plasmoid.include_once("JMultiProgress.class.js"); // JUIProgressMeter inherits this
plasmoid.include_once("JProgressButtons.class.js"); // this.buttons
plasmoid.include_once("3rdparty/sprintf.js"); // for sprintf

/**
 * Constructor.
 *
 * A multiprogress interface that shows progress bars.
 */
function JUIProgressMeter()
{
	// parent constructor
	JMultiProgress.call(this);

	this.type = $.UITypeMeters;

	// init props
	this.rect = new QRectF();
	this.next_row = 1;

	// style that is used when no items are in here
	this.no_items_style = {};
	this.no_items_style.border_color = new QColor("darkgrey");
	this.no_items_style.border_width = 2;

	// buttons
	this.buttons = new JProgressButtons(this, "circle", 48);
}

// prototype inheritance
JUIProgressMeter.inheritsPrototype(JMultiProgress);

/**
 * Initialize a new item with default parameters.
 *
 * @param JProgress item
 */
JUIProgressMeter.prototype.initItem = function(item)
{
	JMultiProgress.prototype.initItem.call(this, item);

	// style goes to ui member
	var ui = item.ui = {};
	ui.color = new QColor("white");
	ui.border_color = new QColor("black");
	ui.border_width = 2;

	var row = this.next_row;

	// create buttons
	this.buttons.initItem(item);
	ui.iw_config_button.minimumSize = new QSizeF(24, 24);
	if (ui.iw_play_button)
		ui.iw_play_button.minimumSize = new QSizeF(24, 24);

	// insert them
	top_layout.addItem(ui.iw_config_button, row, 1);
	if (ui.iw_play_button)
		top_layout.addItem(ui.iw_play_button, row, 2);

	// create label
	ui.label = new Label;
	ui.label.text = item.title;
	ui.label.alignment = QtAlignHCenter | QtAlignVCenter;
	// setup font
	var font = ui.label.font;
	font.bold = true;
	ui.label.font = font;

	// insert it
	top_layout.addItem(ui.label, row, 3);
	// sizePolicy is broken in API v1 so this is a workaround
	// needed tp prevent label to stretch horizontally
	top_layout.setRowMaximumHeight(row, this.buttons.max_button_size);

	row++;

	// add meter and set it up
	ui.meter = new Meter;
	top_layout.addItem(ui.meter, row, 1, 1, 3);
	ui.meter.meterType = ui.meter.BarMeterHorizontal
	ui.meter.minimum = 0;
	ui.meter.maximum = 10000; // this defines precision

	// update widgets on request
	ui.refreshWidgets = function ()
	{
		this.ui.meter.value = Math.round(this.fraction() * this.ui.meter.maximum);
		ui.label.text = item.title + ": " + sprintf("%2.2f%%", Math.round(this.fraction() * 10000) / 100);
	};
	// set initial values
	ui.refreshWidgets.call(item);

	// connect refreshWidgets to progress signals
	if (item.minimumChanged)
		item.minimumChanged.connect(ui.refreshWidgets);
	item.maximumChanged.connect(ui.refreshWidgets);
	item.valueChanged.connect(ui.refreshWidgets);
	item.titleChanged.connect(ui.refreshWidgets);

	this.next_row = row + 1;
}

/**
 * Free all resources of the item before delete.
 *
 * @param JProgress item
 */
JUIProgressMeter.prototype.freeItem = function(item)
{
	var ui = item.ui;

	// disconnect slots
	if (item.minimumChanged)
		item.minimumChanged.disconnect(ui.refreshWidgets);
	item.maximumChanged.disconnect(ui.refreshWidgets);
	item.valueChanged.disconnect(ui.refreshWidgets);
	item.titleChanged.disconnect(ui.refreshWidgets);
	// remove label
	ui.label.deleteLater();
	// remove meter
	ui.meter.deleteLater();
	// remove buttons
	this.buttons.freeItem(item);

	JMultiProgress.prototype.freeItem.call(this, item);
};

/**
 * Export configuration properties of the item.
 *
 * @param JProgress item
 * @return object exported item
 */
JUIProgressMeter.prototype.exportItem = function(item)
{
	var o = item.Export();
	var ui = item.ui;
	o.ui = {};
	o.ui.type = $.UIType.Export(this.type);
	o.ui.color = QColor.Export(ui.color);
	o.ui.border_color = QColor.Export(ui.border_color);
	return o;
};

/**
 * Import configuration properties of the item.
 *
 * @param JProgress item
 * @param object o imported data
 */
JUIProgressMeter.prototype.importItem = function(item, o)
{
	if (o && o.ui)
	{
		var ui = item.ui;
		ui.color = QColor.Import(o.ui.color, ui.color);
		ui.border_color = QColor.Import(o.ui.border_color, ui.border_color);
		item.updated.emit()
	}

	item.Import(o); // this will call update
};

/**
 * Tell widget to draw only within the given rectangle.
 *
 * @param QRectF rect rectangle for drawing widget in
 */
JUIProgressMeter.prototype.fitInRect = function(rect)
{
	// actually, this rect is responsible only for drawing no items case
	this.rect = rect.clone();
};

/**
 * Paint the widget when no items are exist.
 *
 * Draw a circle without any fill and a center point.
 *
 * @param QPainter painter
 */
JUIProgressMeter.prototype.paintNoItems = function(painter)
{
	// setup painter
	var pen = new QPen;
	pen.color = this.no_items_style.border_color;
	pen.width = this.no_items_style.border_width;
	painter.pen = pen;
	painter.setBrushColor(new QColor("transparent"));

	// draw two rectangles
	var prect = this.rect.adjustedUniformly(-pen.width / 2);
	painter.drawRect(prect.x, prect.y, prect.width / 2, prect.height);
	painter.drawRect(prect.x + prect.width / 2, prect.y, prect.width / 2, prect.height);
};

/**
 * Paint the widget.
 *
 * @param QPainter painter
 */
JUIProgressMeter.prototype.paint = function(painter)
{
	var n = this.items.length;
	if (n == 0)
	{
		this.paintNoItems(painter);
		return;
	}
};
