/**
 * Plasma JavaScript API and language extensions.
 *
 * Useful methods and constants are added to standard API classes.
 * Extensions conserned with inheritance and signal-slots.
 *
 * @license GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author Nikita Melnichenko [http://nikita.melnichenko.name]
 * @copyright Copyright 2010, Nikita Melnichenko
 */


// ===== extending QPainter class =====

// enum QPainter::RenderHint
QPainter.Antialiasing = 0x01;
QPainter.TextAntialiasing = 0x02;
QPainter.SmoothPixmapTransform = 0x04;
QPainter.HighQualityAntialiasing = 0x08;
QPainter.NonCosmeticDefaultPen = 0x10;

/**
 * Setup painter to use as max antialiasing as possible.
 */
QPainter.prototype.setFullAntialiasing = function()
{
	var hints = this.renderHints
		| QPainter.Antialiasing
		| QPainter.TextAntialiasing
		| QPainter.SmoothPixmapTransform
		| QPainter.HighQualityAntialiasing;
	this.setRenderHint(hints, true);
};

/**
 * Set brush color for painter.
 *
 * Since there is no QBrush in the JS API v1, this trick helps us to set a brush color.
 * Brush is needed to draw filled oblects like circles.
 *
 * @param QColor color a new brush color
 */
QPainter.prototype.setBrushColor = function(color)
{
	var pen = new QPen;
	pen.color = color;
	this.brush = pen.brush;
};


// ===== extending QRectF class =====

/// determine center of rectangle
QRectF.prototype.center = function()
{
	return new QPoint(this.x + this.width / 2, this.y + this.height / 2);
};

/// clone the rectangle object
QRectF.prototype.clone = function()
{
	return new QRectF(this.x, this.y, this.width, this.height);
};

/**
 * Adjust rectangle uniformly (same size in all directions).
 *
 * @param number size if size > 0 then rectangle grows else it shrinks
 */
QRectF.prototype.adjustUniformly = function(size)
{
	this.adjust(-size, -size, size, size);
};

/**
 * Get uniformly adjusted rectangle (same size in all directions).
 *
 * @param number size if size > 0 then rectangle grows else it shrinks
 */
QRectF.prototype.adjustedUniformly = function(size)
{
	return this.adjusted(-size, -size, size, size);
};


// ===== extending QColor class =====

/**
 * Export properties to a javascript array.
 *
 * The first letter is capital because 'export' gives syntax error (bindings issue?).
 *
 * @param QColor color
 */
QColor.Export = function(color)
{
	return [ color.red, color.green, color.blue ];
};

/**
 * Import properties from the javascript array.
 *
 * The first letter is capital because 'import' gives syntax error (bindings issue?).
 *
 * @param array array list of three values (RGB)
 * @param QColor fallback_value the value to return if array is not valid
 */
QColor.Import = function(array, fallback_value)
{
	if (typeof array === "undefined"
		|| typeof array[0] === "undefined"
		|| typeof array[1] === "undefined"
		|| typeof array[2] === "undefined")
		return fallback_value;
	return new QColor(array[0], array[1], array[2]);
};


// ===== extending plasmoid object =====

/**
 * This function behaves similar to the include() statement,
 * with the only difference being that if the code from a file has already been included,
 * it will not be included again.
 */
plasmoid.include_once = function(filename)
{
	if (plasmoid._included_files[filename] == undefined)
		plasmoid._included_files[filename] = plasmoid.include(filename);
};
/// special variable used by include_once to save which files were included
plasmoid._included_files = {};


// ===== inheritance emulation =====

/**
 * Clone javascript object.
 *
 * Doesn't work on Qt binded classes.
 *
 * @param mixed source any object or variable
 * @param bool own_props_only flag: clone only own properties
 */
function clone(source, own_props_only)
{
	// plain objects are cloned automatically
	if (!(source && typeof source == "object"))
		return source;

	// clone object/array
	var copy = typeof source.pop === "function" ? [] : {};
	for (var prop in source)
		if (!own_props_only || source.hasOwnProperty(prop)) // clone only own props if requested
		{
			// check if getter+setter are set and copy them
			var getter = source.__lookupGetter__(prop);
			var setter = source.__lookupSetter__(prop);
			if (getter && setter)
			{
				copy.__defineGetter__(prop, getter);
				copy.__defineSetter__(prop, setter);
			}
			// otherwise simply clone the member
			else
				copy[prop] = clone(source[prop], own_props_only);
		}
	return copy;
}

/**
 * Inherit prototype from another class.
 *
 * Constructor of the parent class should be called explicitly
 * in the constructor of the current class.
 *
 * @param class parent_class parent class from which we need a prototype
 */
Function.prototype.inheritsPrototype = function(parent_class)
{
	this.prototype = clone(parent_class.prototype, true);
	this.prototype.constructor = this;
};


// ===== enum sugar =====

/**
 * Define enum and import/export methods for it.
 *
 * Example:
 * a.Enum(Direction, { "Up": 0, "Down": 1});
 * gives:
 * a.DirectionUp == 0
 * a.DirectionDown == 1
 * a.Direction.Export(1) == "Down"
 * a.Direction.Import("Up", a.DirectionDown) == 0
 * a.Direction.Import("Invalid", a.DirectionDown) == 1
 *
 * The first letter is capital because 'enum' gives syntax error (bindings issue?).
 * The same is for 'import' and 'export'.
 *
 * @param string name enum name; the name of the member that would contain Import/Export methods
 * @param object values key-value pairs that define enum
 */
Function.prototype.Enum = function(name, values)
{
	// define enum object
	this[name] = {};
	// define constants
	for (var i in values)
		this[name + i] = values[i];
	// getter: name by value
	this[name].Export = function (value)
	{
		for (var i in values)
			if (values[i] == value)
				return i;
	};
	// getter: value by name
	this[name].Import = function (name, fallback_value)
	{
		if (typeof name === "undefined" || typeof values[name] === "undefined")
			return fallback_value;
		return values[name];
	};
};


// ===== import of internal types =====

/**
 * Import an int from string.
 *
 * @param mixed value the value for convertation
 * @param int fallback_value what to return if value is not valid
 */
function importInt(value, fallback_value)
{
	if (typeof value !== "string" && typeof value !== "number")
		return fallback_value;
	var res = parseInt(value);
	if (isNaN(res))
		return fallback_value;
	return res;
}

/**
 * Import a float from string.
 *
 * @param mixed value the value for conversion
 * @param float fallback_value what to return if value is not valid
 */
function importFloat(value, fallback_value)
{
	if (typeof value === "number")
		return value;
	if (typeof value !== "string")
		return fallback_value;
	var res = parseFloat(value);
	if (isNaN(res))
		return fallback_value;
	return res;
}

/**
 * Import a string.
 *
 * @param mixed value the imported value
 * @param string fallback_value what to return if value is not valid
 */
function importString(value, fallback_value)
{
	if (typeof value !== "string")
		return fallback_value;
	return value;
}

/**
 * Import a boolean.
 *
 * @param mixed value the imported value
 * @param boolean fallback_value what to return if value is not valid
 */
function importBoolean(value, fallback_value)
{
	if (typeof value !== "boolean")
		return fallback_value;
	return value;
}

// ===== signals and slots pattern emulation =====

/**
 * Signal object that emulates behaviour of signals from bindings.
 *
 * Typical usage:
 * Put initialization into the constructor: this.mysignal = new JSignal(this);
 * Then connect slot to the signal: obj.mysignal.connect(myfunction);
 * myfunction will receive obj as "this" object.
 */
function JSignal(source)
{
	// signal owner
	this.source = source;
	// connected slots
	this.slots = [];
}

/**
 * Connect slot to the signal.
 *
 * @param function slot
 */
JSignal.prototype.connect = function(slot)
{
	if (typeof slot === "function")
		this.slots.push(slot);
	else
		print("[JSignal:connect] Slot is not a function.");
};

/**
 * Disconnect slot to the signal.
 *
 * If slot isn't used then disconnect all slots.
 *
 * @param function slot
 */
JSignal.prototype.disconnect = function(slot)
{
	// if no slot is set then disconnect all slots
	if (!slot)
	{
		this.slots = [];
		return;
	}

	// check that slot is a function
	if (typeof slot !== "function")
		print("[JSignal:disconnect] Slot is not a function.");

	// search and delete the slot
	for (var i in this.slots)
		if (this.slots[i] === slot)
		{
			this.slots.splice(i, 1);
			return;
		}

	print("[JSignal:disconnect] The slot is not connected to the signal.");
};

/**
 * Emit the signal.
 *
 * This means all slots will be call one by one.
 * Supply any number of parameters, they will be passed to each slot.
 */
JSignal.prototype.emit = function()
{
	for (var i = 0; i < this.slots.length; i++)
		this.slots[i].apply(this.source, arguments);
};

/**
 * Slot is not an object, it's a function generator.
 *
 * Typical usage:
 * this.myslot = JSlot(this, this.func);
 * myslot() call is static and equivalent to this.func()
 */
function JSlot($this, func)
{
	return function () { func.apply($this, arguments); };
}
