/**
 * Helper functions for running different external scripts and applications.
 *
 * @license GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author Nikita Melnichenko [http://nikita.melnichenko.name]
 * @copyright Copyright 2010, Nikita Melnichenko
 */

/// find a path to a bundled script by its name
function scriptPath (name)
{
	// it's a hack to find a file via ui type, but empty type doesn't work and not documented on 4.4.1
	return plasmoid.file("ui", "../scripts/" + name);
}

/**
 * Escape a string to be used as a shell argument.
 *
 * This is a port of php function of the same name (note that phpjs.org has wrong port).
 * Example: "it's a good day' -> "'it'\''s a good day'"
 *
 * @param string arg
 */
function escapeshellarg(arg)
{
	// creating a RegExp object to avoid highlighting problems ;)
	var regexp = new RegExp("[^\\\\]'", "g");
	var ret = arg.replace(regexp, function(m, i, s)
	{
		return m.slice(0, 1) + "'\\''";
	});
	return "'" + ret + "'";
}

/**
 * Compose full command line that could be used in dataEngine("executable").
 *
 * @param string command
 * @param array args command arguments, array of (non-escaped) strings
 */
function commandLine(command, args)
{
	var command_line = command;
	for (var i in args)
		command_line += " " + escapeshellarg(args[i]);
	return command_line;
}

/**
 * Run command line and forget.
 *
 * Use "executable" engine to run a command line.
 *
 * @param string line command line
 */
function runCommandLine(line)
{
	if (!line)
		return false;

	// creating a fake object to connectSource to it (can't connect to the plasmoid, cause it has its own dataUpdated)
	// among many available widgets Svg seems to be the best choice - it doesn't show itself anywhere
	// disadvantage is that we have "No such slot" warning on console
	var fake_object = new Svg("");
	fake_object.dataUpdated = function(source, data) {};
	dataEngine("executable").connectSource(line, fake_object);
	return true;
}

/**
 * Run command and forget.
 *
 * Try to run command with LaunchApp extension or use "executable" engine.
 *
 * @param string command
 * @param array args command arguments, array of strings
 */
function runCommand(command, args)
{
	if (!command)
		return false;
	if (!args)
		args = [];

	// try with LaunchApp or use an alternative way with executable data engine
	if (plasmoid.hasExtension("LaunchApp"))
		return plasmoid.runCommand(command, args);

	return runCommandLine(commandLine(command, args))
}

/**
 * Show a message via KNotify system.
 *
 * @param KNotify.Type type message type, one of "notification", "warning" or "error"
 * @param string message displayed message
 * @param int timeout timeout is milliseconds
 */
function KNotify(type, message, timeout)
{
	// find a path to the knotify script
	var script = scriptPath ("knotify.py");
	// call
	return runCommand("python", [script, KNotify.Type.Export(type), plasmoid.title, message, timeout]);
};

// define Type enum
KNotify.Enum("Type",
{
	"Notification": 0,
	"Warning": 1,
	"Error": 2
});
