/**
 * Main component of the plasmoid Make Progress!.
 *
 * Required dependencies: KDE >= 4.4.1.
 * Highly recommended dependencies: PyQt >= 4.7 (that's for Qt 4.6), PyKDE.
 * Optional dependencies: Make Progress data engine.
 *
 * PyQt is needed for dialogs. PyKDE is not used explicitly but is possibly needed for KDE widgets there.
 * If PyQt is not available you can manage items only from DBus (if you have data engine installed, of course).
 * Currently the data engine is also use PyQt so you're out of options now, but that's a separate component,
 * so it could be rewritten by someone who wants to get rid of this dep.
 * Well, the facts say that it's a required dependency today...
 * Sorry for inconvenience but JS bindings (API v1 that's in KDE 4.4.1) are in the early stage of development
 * and have no support for custom dialogs at all. Once it would be there we gonna rewrite this part.
 *
 * Thanks to:
 * - Aaron Seigo <aseigo[at]kde.org>, who was patient and asked many questions concerning the API
 *   and also fixed bugs in JS bindings as they appear.
 *
 * @license GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author Nikita Melnichenko [http://nikita.melnichenko.name]
 * @copyright Copyright 2010, Nikita Melnichenko
 */

// API extensions must be always the first include file
plasmoid.include("api_extensions.js");

// the name of plasmoid (used in messages, notifications, etc.)
plasmoid.title = i18n("Make progress!");

// create the central object
$ = {};

// define static properties and functions of it

/// debug flag
$.debug = false;

/// console message function
$.message = function(text)
{
	print("[" + plasmoid.title + "] " + text);
};

/// debug print
$.debug_print = function(text)
{
	if ($.debug)
		print(text);
};

// allow the central object to have enums
$.Enum = Function.prototype.Enum;

// define ProgressType enum
$.Enum("ProgressType",
{
	"Basic": 0,
	"Live": 1,
	"Timer": 2,
	"Poll": 3
});

// define UIType enum
// NOTE: this mapping must be synced with a config dialog combo
$.Enum("UIType",
{
	"Circles": 0,
	"Bars": 1,
	"Meters": 2
});

// include remaining stuff

// progresses (they use $.ProgressType)
plasmoid.include_once("JProgress.class.js");
plasmoid.include_once("JLiveProgress.class.js");
plasmoid.include_once("JTimer.class.js");
plasmoid.include_once("JPollProgress.class.js");
// UIs (they use $.UIType)
plasmoid.include_once("JUIProgressCircle.class.js");
plasmoid.include_once("JUIProgressBar.class.js");
plasmoid.include_once("JUIProgressMeter.class.js");
// DBus bridge
plasmoid.include_once("JDBus.class.js");
// debugging
plasmoid.include_once("3rdparty/var_dump.js"); // for var_dump in plasmoid.dataUpdated


// universal function for creating progresses and UIs by type

/**
 * Create a new progress.
 *
 * @param $.ProgressType type
 * @return JProgress a new progress that is derived from JProgress
 */
$.newProgress = function(type)
{
	// NOTE: this mapping must be synced with types in class contructors
	switch (type)
	{
		case $.ProgressTypeBasic:
			return new JProgress();
		case $.ProgressTypeLive:
			return new JLiveProgress();
		case $.ProgressTypeTimer:
			return new JTimer();
		case $.ProgressTypePoll:
			return new JPollProgress();
	}
};

/**
 * Create a new multiprogress UI.
 *
 * @param $.UIType type
 * @return JMultiProgress a new widget that is derived from JMultiProgress
 */
$.newUI = function(type)
{
	// NOTE: this mapping must be synced with types in class contructors
	switch (type)
	{
		case $.UITypeCircles:
			return new JUIProgressCircle();
		case $.UITypeBars:
			return new JUIProgressBar();
		case $.UITypeMeters:
			return new JUIProgressMeter();
	}
};

/// init the central object and the plasmoid from config
$.configInit = function()
{
	$.debug_print("@@@ $.configInit");

	// define name property
	$.__defineGetter__("name", function()
	{
		return plasmoid.readConfig("name");
	});
	$.__defineSetter__("name", function(value)
	{
		$.debug_print("@@@ $.name: writing to the config");
		plasmoid.writeConfig("name", value);
	});
	// print initial name
	$.message("My name is '" + $.name + "'");

	// create the main interface object
	$.i = $.newUI(plasmoid.readConfig("interface"));
	// tell widget to fit in plasmoid
	$.i.fitInRect(plasmoid.rect);
	// redraw plasmoid when widget is updated
	$.i.updated.connect(plasmoid.update);

	// create defaults container
	$.defaults = {};

	// load some properties in the same way plasmoid.configChanged do
	$.reconfigure();

	// load items from config
	var json_items = plasmoid.readConfig("items");
	$.i.importItems(JSON.parse(json_items));
	$.counter += $.i.items.length;

	// schedule items for saving when widget is updated
	$.i.updated.connect($.saveItems);
};

/**
 * Save items into configuration file.
 *
 * It should be called on every data update.
 * Internal mechanism prevents writing of config too often.
 */
$.saveItems = function()
{
	// minimum interval between writes
	var interval = 1000; // 1 second

	// create a timer if it wasn't created before
	if (!$.saveItems_timer)
	{
		$.saveItems_timer = new QTimer();
		$.saveItems_timer.singleShot = true;
		$.saveItems_timer.timeout.connect($.saveItems);
	}

	// compute a delay
	var delay = 0;
	if ($.saveItems_date)
	{
		var current_date = new Date();
		var msecs = current_date - $.saveItems_date;
		delay = Math.max(0, interval - msecs)
	}

	// wait or save config now
	if (delay)
	{
		$.debug_print("@@@ $.saveItems: delayed for " + delay + " msecs");
		if (!$.saveItems_timer.active)
			$.saveItems_timer.start(delay);
	}
	else
	{
		$.debug_print("@@@ $.saveItems: writing to the config");
		plasmoid.writeConfig("items", JSON.stringify($.i.exportItems()));
		$.saveItems_timer.stop();
		$.saveItems_date = new Date();
	}
};

/// Init some properties of $ from config that could be reloaded any time.
$.reconfigure = function()
{
	// manage buttons
	if ($.i.buttons)
	{
		if (typeof $.i.buttons.setConfigButtonsVisibility === "function")
			$.i.buttons.setConfigButtonsVisibility(plasmoid.readConfig("isConfigButtonShown"));
		if (typeof $.i.buttons.setPlayButtonsVisibility === "function")
			$.i.buttons.setPlayButtonsVisibility(plasmoid.readConfig("isPlayButtonShown"));
	}

	// set theme
	$.theme = $.themes[$.Theme.Export(plasmoid.readConfig("theme"))];

	// read default action
	$.defaults.actionOnFinish = new JAction();
	$.defaults.actionOnFinish.Import(
	{
		use_knotify: plasmoid.readConfig("actionUseKNotify"),
		knotify_type: KNotify.Type.Export(plasmoid.readConfig("actionKNotifyType")),
		knotify_template: plasmoid.readConfig("actionKNotifyTemplate"),
		knotify_timeout: plasmoid.readConfig("actionKNotifyTimeout"),
		use_command: plasmoid.readConfig("actionUseCommand"),
		command_line: plasmoid.readConfig("actionCommand")
	});

	// plasmoid can be freely resized
	plasmoid.aspectRatioMode = IgnoreAspectRatio;

	// correct depend on form factor: minimum size, background, etc
	plasmoid.formFactorChanged();

	// plasmoid size can change if background is changed
	plasmoid.sizeChanged();
};

/* Predefined color themes for new items.
 *
 * Theme is array of parameter sets which is imported to new item.
 */
$.themes =
{
	BlackAndWhite:
	[
		{
			color: [ 255, 255, 255 ], // white
			border_color: [ 0, 0, 0 ], // black
			background_color: [ 211, 211, 211 ], // lightgrey
			background_border_color: [ 169, 169, 169 ], // darkgrey
		}
	],
	GreyZebra:
	[
		{
			color: [ 255, 250, 250 ], // snow
			border_color: [ 80, 80, 80 ],
			background_color: [ 221, 221, 221 ], // lighter than lightgrey
			background_border_color: [ 169, 169, 169 ], // darkgrey
		},
		{
			color: [ 128, 128, 128 ], // grey
			border_color: [ 50, 50, 50 ],
			background_color: [ 201, 201, 201 ], // darker than lightgrey
			background_border_color: [ 169, 169, 169 ], // darkgrey
		},
	],
	Rainbow:
	[
		{
			color: [ 205, 92, 92 ], // indianred
			border_color: [ 139, 0, 0 ], // darkred
			background_color: [ 231, 211, 211 ],
			background_border_color: [ 169, 169, 169 ], // darkgrey
		},
		{
			color: [ 244,164,96 ], // sandybrown
			border_color: [ 210,105,30  ], // chocolate
			background_color: [ 231, 221, 201 ],
			background_border_color: [ 169, 169, 169 ], // darkgrey
		},
		{
			color: [ 255,215,0 ], // gold
			border_color: [ 218,165,32  ], // goldenrod
			background_color: [ 231, 231, 201 ],
			background_border_color: [ 169, 169, 169 ], // darkgrey
		},
		{
			color: [ 107,142,35 ], // olivedrab
			border_color: [ 0,100,0  ], // darkgreen
			background_color: [ 211, 231, 211 ],
			background_border_color: [ 169, 169, 169 ], // darkgrey
		},
		{
			color: [ 135,206,235 ], // lightskyblue
			border_color: [ 30,144,255 ], // dodgerblue
			background_color: [ 211, 221, 231 ],
			background_border_color: [ 169, 169, 169 ], // darkgrey
		},
		{
			color: [ 65,105,225 ], // royalblue
			border_color: [ 25,25,112  ], // midnightblue
			background_color: [ 211, 211, 231 ],
			background_border_color: [ 169, 169, 169 ], // darkgrey
		},
		{
			color: [ 106,90,205 ], // slateblue
			border_color: [ 75,0,130  ], // indigo
			background_color: [ 221, 211, 231 ],
			background_border_color: [ 169, 169, 169 ], // darkgrey
		},
	],
};

// define theme enum
$.Enum("Theme",
{
	"BlackAndWhite": 0,
	"GreyZebra": 1,
	"Rainbow": 2
});

/// number of created progresses in the current session (used for theme)
$.counter = 0;

/**
 * Apply theme on a progress.
 *
 * @param JProgress item progress
 */
$.applyTheme = function(item)
{
	if ($.theme)
	{
		var theme_ui = $.theme[$.counter % $.theme.length];
		$.i.importItem(item, { ui: theme_ui });
	}
	$.counter++;
};

/**
 * Create a new progress, add it to a multiprogress, apply current theme.
 *
 * @param $.ProgressType type
 * @return JProgress a new progress that is derived from JProgress
 */
$.createProgress = function(type)
{
	// create and add item
	var item = $.i.addItem($.newProgress(type));
	$.message("New " + $.ProgressType.Export(type) + " progress is created.");
	// apply theme
	$.applyTheme(item);
	// register in DBus
	$.jdbus.registerItem(item);
	return item;
};

/// Callback: paint event.
plasmoid.paintInterface = function(painter)
{
	painter.setFullAntialiasing();
	$.i.paint(painter);
};

/// Callback: plasmoid was resized.
plasmoid.sizeChanged = function()
{
	$.i.fitInRect(plasmoid.rect);
	plasmoid.update();
};

/// Callback: plasmoid form factor is changed.
plasmoid.formFactorChanged = function()
{
	if (plasmoid.formFactor == Planar)
	{
		// set background if plasmoid is on a desktop
		switch(plasmoid.readConfig("background"))
		{
			case 0:
				plasmoid.backgroundHints = NoBackground;
				break;
			case 1:
				plasmoid.backgroundHints = StandardBackground;
				break;
			case 2:
				plasmoid.backgroundHints = TranslucentBackground;
				break;
		}

		// set minimum size depend on background
		// the size in setMinimumSize takes count of background borders!
		if (plasmoid.backgroundHints == NoBackground)
			plasmoid.setMinimumSize(20, 20);
		else
			plasmoid.setMinimumSize(40, 40);
	}
	else
	{
		// on a panel always turn off background to get more space for drawing
		plasmoid.backgroundHints = NoBackground;
		// plasmoids on panels could be rather small
		plasmoid.setMinimumSize(10, 10);
		// set Preferred Size so plasmoid would not be too small
		// it's not clear how to resize plasmoids on panels... :(
		plasmoid.setPreferredSize(50, 50);
	}
};

/// Callback: configuration was changed.
plasmoid.configChanged = function()
{
	$.debug_print("@@@ plasmoid.configChanged");

	// check if UI type is changed
	// and if it does, try to switch to the new ui type and save maximum number of common properties
	var ui_type = plasmoid.readConfig("interface");
	if (ui_type != $.i.type)
	{
		// save an old widget
		var $i = $.i;
		// create a new widget
		$.i = $.newUI(ui_type);
		// tell widget to fit in plasmoid
		$.i.fitInRect(plasmoid.rect);
		// redraw plasmoid when widget is updated
		$.i.updated.connect(plasmoid.update);
		// process item by item and add them to new widget
		for (var i in $i.items)
		{
			var item = $i.items[i];
			// export ui properties
			var ui = $i.exportItem(item).ui;
			var is_active = item.active; // this helps to keep item running cause freeItem unconditionally stops it
			// remove supporting elements
			$i.freeItem(item)
			// add to a new widget
			$.i.addItem(item);
			// import ui properties (method desides which is right for widget)
			$.i.importItem(item, { ui: ui, active: is_active });
		}
		// schedule items for saving when widget is updated
		$.i.updated.connect($.saveItems);
		// copy scheduleItemRemoval that could be changed by jdbus
		$.i.scheduleItemRemoval = $i.scheduleItemRemoval;
		// disconnect old widget
		$i.updated.disconnect();
		// update a new widget
		$.i.update();
	}

	// reload other properties
	$.reconfigure();
};

/**
 * dataUpdated slot for events from different data engines.
 *
 * JS API v1 allows to connect source to plasmoid.dataUpdated only.
 * To separate engine replies from each other some tricks were used.
 * See also JDBus.dataUpdated and JDialog.dataUpdated.
 *
 * @param string source name of the source
 * @param string data source data
 */
plasmoid.dataUpdated = function(source, data)
{
	if ($.debug)
	{
		print("@@@ plasmoid.dataUpdated for source '" + source + "' and data:");
		var_dump(data);
	}
	// check if that's JDBus source
	if ($.jdbus && $.jdbus.dataUpdated(source, data))
		return;
	// check if that's JDialog source
	if ($.i.dialog && $.i.dialog.dataUpdated(source, data))
		return;
	// check if some of items wait for this source
	var item_handled = false;
	for(var i in $.i.items)
		if ($.i.items[i].dataUpdated)
		{
			var is_handled = $.i.items[i].dataUpdated(source, data);
			if (is_handled)
				item_handled = true;
		}
	if (item_handled)
		return;
	// shouldn't pass here
	$.debug_print("@@@ Warning: unhandled case in plasmoid.dataUpdated");
};

// define actions (appeared in popup menu)

plasmoid.action_NewProgress = function()
{
	$.createProgress($.ProgressTypeBasic);
};

plasmoid.action_NewLiveProgress = function()
{
	$.createProgress($.ProgressTypeLive);
};

plasmoid.action_NewTimer = function()
{
	$.createProgress($.ProgressTypeTimer);
};

plasmoid.action_NewPoll = function()
{
	$.createProgress($.ProgressTypePoll);
};

plasmoid.setAction("NewProgress", i18n("New progress"), "list-add")
plasmoid.setAction("NewLiveProgress", i18n("New live progress"), "list-add")
plasmoid.setAction("NewTimer", i18n("New timer"), "chronometer")
plasmoid.setAction("NewPoll", i18n("New poll"), "applications-system")

// top level layout, used in Meters UI
// must be created only once or plasmoid will freak out
top_layout = new GridLayout;


// init the central object and the plasmoid from config
$.configInit();

// create a DBus bridge
$.jdbus = new JDBus;
// when jdbus is activated we would replace scheduleItemRemoval function of multiprogress to the one that connects to the engine
$.jdbus.activated.connect(function () { $.i.scheduleItemRemoval = function (item) { $.jdbus.scheduleItemRemoval(item); }; });
// launch registration procedure
$.jdbus.register();
